@@ -61,7 +61,7 @@
         [clojure.tools.nrepl.transport :only (tty tty-greeting)]
         [clojure.core.incubator :only (-?>)]
         [com.puppetlabs.jdbc :only (with-transacted-connection)]
-        [com.puppetlabs.utils :only (cli! configure-logging! inis-to-map with-error-delivery)]
+        [com.puppetlabs.utils :only (cli! configure-logging! inis-to-map with-error-delivery version update-info)]
         [com.puppetlabs.puppetdb.scf.migrate :only [migrate!]]))
 
 (def cli-description "Main PuppetDB daemon")
@@ -71,7 +71,6 @@
 ;; The following functions setup interaction between the main
 ;; PuppetDB components.
 
-(def version (pl-utils/get-version-from-manifest))
 (def configuration nil)
 (def mq-addr "vm://localhost?jms.prefetchPolicy.all=1&create=false")
 (def mq-endpoint "com.puppetlabs.puppetdb.commands")
@@ -121,6 +120,21 @@
 
      (sleep))))
 
+(defn check-for-updates
+  "This will fetch the latest version number of PuppetDB and log if the system
+  is out of date."
+  [update-server]
+  (let [{:keys [version newer link]} (try
+                                       (update-info update-server)
+                                       (catch Throwable e
+                                         (log/debug e "Could not retrieve update information")))
+        link-str                     (if link
+                                       (format "Visit %s for details." link)
+                                       "")
+        update-msg                   (format "Newer version %s is available! %s" version link-str)]
+    (when newer
+      (log/info update-msg))))
+
 (defn configure-commandproc-threads
   "Update the supplied config map with the number of
   command-processing threads to use. If no value exists in the config
@@ -197,22 +211,29 @@
   (def configuration config)
   config)
 
-(defn parse-config
+(defn parse-config!
   "Parses the given config file/directory and configures its various
-  subcomponents."
-  [path]
-  {:pre [(string? path)]}
-  (let [file (file path)]
-    (when-not (.canRead file)
-      (throw (IllegalArgumentException.
-              (format "Configuration path '%s' must exist and must be readable." path)))))
+  subcomponents.
 
-  (-> (inis-to-map path)
-      (configure-logging!)
-      (configure-commandproc-threads)
-      (configure-web-server)
-      (configure-database)
-      (set-global-configuration!)))
+  Also accepts an optional map argument 'initial-config'; if
+  provided, any initial values in this map will be included
+  in the resulting config map."
+  ([path]
+    (parse-config! path {}))
+  ([path initial-config]
+    {:pre [(string? path)
+           (map? initial-config)]}
+    (let [file (file path)]
+      (when-not (.canRead file)
+        (throw (IllegalArgumentException.
+                (format "Configuration path '%s' must exist and must be readable." path)))))
+
+    (-> (merge initial-config (inis-to-map path))
+        (configure-logging!)
+        (configure-commandproc-threads)
+        (configure-web-server)
+        (configure-database)
+        (set-global-configuration!))))
 
 (defn on-shutdown
   "General cleanup when a shutdown request is received."
@@ -224,8 +245,9 @@
   [& args]
   (let [[options _]                                (cli! args)
         initial-config                             {:debug (:debug options)}
-        {:keys [jetty database global] :as config} (merge initial-config (parse-config (:config options)))
+        {:keys [jetty database global] :as config} (parse-config! (:config options) initial-config)
         vardir                                     (validate-vardir (:vardir global))
+        update-server                              (:update-server global "http://updates.puppetlabs.com/check-for-updates")
         resource-query-limit                       (get global :resource-query-limit 20000)
         db                                         (pl-jdbc/pooled-datasource database)
         db-gc-minutes                              (get database :gc-interval 60)
@@ -235,12 +257,13 @@
         globals                                    {:scf-db               db
                                                     :command-mq           {:connection-string mq-addr
                                                                            :endpoint          mq-endpoint}
-                                                    :resource-query-limit resource-query-limit}]
+                                                    :resource-query-limit resource-query-limit
+                                                    :update-server        update-server}]
 
 
 
-    (when version
-      (log/info (format "PuppetDB version %s" version)))
+    (when (version)
+      (log/info (format "PuppetDB version %s" (version))))
 
     ;; Add a shutdown hook where we can handle any required cleanup
     (pl-utils/add-shutdown-hook! on-shutdown)
@@ -261,6 +284,7 @@
                           (vec (for [n (range nthreads)]
                                  (future (with-error-delivery error
                                            (load-from-mq mq-addr mq-endpoint discard-dir db))))))
+          updater       (future (check-for-updates update-server))
           web-app       (let [authorized? (if-let [wl (jetty :certificate-whitelist)]
                                             (pl-utils/cn-whitelist->authorizer wl)
                                             (constantly true))
@@ -287,6 +311,7 @@
       (let [exception (deref error)]
         (doseq [cp command-procs]
           (future-cancel cp))
+        (future-cancel updater)
         (future-cancel web-app)
         (future-cancel db-gc)
         ;; Stop the mq the old-fashioned way
